package m3uebung03;

import java.applet.*;
import java.awt.*;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.LinkedList;

import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

public class DGLDemo extends Applet {

	private static final long serialVersionUID = 1L;
	
	double stoerx = 5, stoery = 5;
	int space = 0;

	private RenderThread render;
	
	JSlider sliderM, sliderR, sliderH;
	JPanel panel = new JPanel(new FlowLayout());

	// The width and height of our offscreen image
	private int width = -1;

	private int height = -1;

	// The offscreen image
	private Image offscreen;

	// switch: are we double buffering or not?
	private boolean dbon = true;

	// Use this to turn double buffering on and off
	protected void setDoubleBuffering(boolean dbon) {
		this.dbon = dbon;
		if (!dbon) {
			offscreen = null;
		}
	}

	// Depending on the value of our switch, we either call our
	// special code, or just call the default code
	public void update(Graphics g) {
		if (dbon) {
			updateDoubleBufffered(g);
		} else {
			super.update(g);
		}
	}

	// Do the drawing to an offscreen buffer -- maybe
	private void updateDoubleBufffered(Graphics g) {

		// Let's make sure we have an offscreen buffer, and that
		// it's the right size. If the applet has been resized,
		// our buffer will be the wrong size and we need to make
		// a new one
		Dimension d = getSize();
		if (offscreen == null || width != d.width || height != d.height
				|| offscreen == null) {
			width = d.width;
			height = d.height;
			if (width > 0 || height > 0) {
				offscreen = createImage(width, height - space);
			} else
				offscreen = null;
		}

		// If we still don't have one, give up
		if (offscreen == null)
			return;

		// Get the off-screen graphics object
		Graphics gg = offscreen.getGraphics();

		// Clear the off-screen graphics object
		gg.setColor(getBackground());
		gg.fillRect( 0, 0, width, height );
		gg.setColor(getForeground());

		// Draw to the off-screen graphics object
		paint(gg);

		// We don't need this Graphics object anymore
		gg.dispose();

		// Finally, we transfer the newly-drawn stuff right to the
		// screen
		g.drawImage(offscreen, 0, space, null);
	}

	public void init() {
		
		super.init();
		
		this.addMouseListener(new MouseListener(){
		
			public void mouseReleased(MouseEvent arg0) {
				render.pause();
				
			}
		
			public void mousePressed(MouseEvent arg0) {
				render.unpause();
				
			}
		
			public void mouseExited(MouseEvent arg0) {
				// TODO Auto-generated method stub
				
			}
		
			public void mouseEntered(MouseEvent arg0) {
				// TODO Auto-generated method stub
				
			}
		
			public void mouseClicked(MouseEvent arg0) {
				// TODO Auto-generated method stub
				
			}
		
		});

		this.addMouseMotionListener(new MouseMotionListener() {
		
			public void mouseMoved(MouseEvent e) {
				
			}
		
			public void mouseDragged(MouseEvent e) {
				stoerx = e.getX();
				stoery = e.getY();
			}
		
		});
		
		sliderM = new JSlider(JSlider.HORIZONTAL, 0, 100, 50);
		sliderM.setSize(200, space);
		sliderM.addChangeListener(new ChangeListener(){
		
			public void stateChanged(ChangeEvent e) {
				 updateConst();
			}		
		});
		sliderM.setToolTipText("Mass");
		
		sliderR = new JSlider(JSlider.HORIZONTAL, 0, 100, 50);
		sliderR.setSize(200, space);
		sliderR.addChangeListener(new ChangeListener(){
		
			public void stateChanged(ChangeEvent e) {;
				updateConst();
			}		
		});
		sliderR.setToolTipText("Friction");
		
		sliderH = new JSlider(JSlider.HORIZONTAL, 0, 100, 50);
		sliderH.setSize(200, space);
		sliderH.addChangeListener(new ChangeListener(){
		
			public void stateChanged(ChangeEvent e) {
				updateConst();
			}		
		});
		sliderH.setToolTipText("h, the time step size");
		
		panel.add(sliderM);
		panel.add(sliderR);
		panel.add(sliderH);
		
		super.add(panel);
		
		updateConst();
		
		render = new RenderThread(this);
		render.start();
		
	}
	
	public void paint(Graphics g) {
		
		super.paint(g);
		
		int s = 8;
		
		g.setColor(Color.RED);
	    for (Vec2 e : xy) {
	    	g.fillRect((int) (e.x * 20.d), (int) (e.y * 20.d), s, s);
		}
	    
	    g.setColor(Color.BLACK);
		g.fillRect((int) stoerx, (int) stoery, s, s);
		
	}
	
	//change conditional parameters
	protected void updateConst() {
		h = sliderH.getValue()/200.d;
		c1 = (sliderR.getValue()/200.d)/(sliderM.getValue()/50.d);
		c2 = 1;
		A[1][0] = -c2; 
		A[1][1] = -c1;
	}

	public void stop(){
		render.disable();
	}
	
	//multiply matrix*vector
	private double[] mul(double[][] A, double[] x, int n){
		
		double ret[] = new double[n];
		
		for (int i = 0; i < n; i++) {
			ret[i] = 0;
			for (int j = 0; j < n; j++) {
				ret[i] += A[i][j]*x[j];
			}
		}
		
		return ret;
	}
	
	//multiply vector*scalar
	private double[] mul(double[] a, double x, int n){
		
		double ret[] = new double[n];
		
		for (int i = 0; i < n; i++) {
			ret[i] = a[i]*x;
		}
		
		return ret;
	}
	
	//add vector+vector
    private double[] add(double[] a, double[] b, int n){
		
		double ret[] = new double[n];
		
		for (int i = 0; i < n; i++) {
			ret[i] = a[i] + b[i];
		}
		
		return ret;
	}
	
	
	private double stoerX(double t){
		return stoerx/20.d;
	}
	
	private double stoerY(double t){
		return stoery/20.d;
	}
	
	//Runge-Kutta for x''(t) + c1 * x'(t) + c2 * x(t) = stoer(t)
	
	double h = 0.5;
	int k = 0;
	double masse = 1.2;
	double reibung = 0.6;
	
	double c1 = reibung/masse;
	double c2 = 1;
	
	double A[][] = {{0, 1}, {-c2, -c1}};
	
	double xnull[] = {0, 0};
	double ynull[] = {0, 0};
	
	LinkedList<Vec2> xy = new LinkedList<Vec2>();
	
	double[] k1, k2, k3, k4; 
	double[] s = new double[2];
	double[] w = new double[2];
	
	public void doit() {
		
		doitX();
		doitY();
		k++;
		
	}
	
	public void doitX() {

		k1 = mul(A, xnull, 2);
		k1[1] += stoerX(h * k);

		k2 = mul(A, add(xnull, mul(k1, 0.5 * h, 2), 2), 2);
		k2[1] += stoerX(h * k + h / 2);

		k3 = mul(A, add(xnull, mul(k2, 0.5 * h, 2), 2), 2);
		k3[1] += stoerX(h * k + h / 2);

		k4 = mul(A, add(xnull, mul(k3, h, 2), 2), 2);
		k4[1] += stoerX(h * k + h);

		for (int i = 0; i < 2; i++) {
			s[i] = k1[i] + 2 * k2[i] + 2 * k3[i] + k4[i];
		}

		w = add(xnull, mul(s, h / 6, 2), 2);
		xnull = w;

		xy.addLast(new Vec2(w[0], 0));
		if (xy.size()>100){
			xy.removeFirst();
		}
	}
	
	public void doitY() {

		k1 = mul(A, ynull, 2);
		k1[1] += stoerY(h * k);

		k2 = mul(A, add(ynull, mul(k1, 0.5 * h, 2), 2), 2);
		k2[1] += stoerY(h * k + h / 2);

		k3 = mul(A, add(ynull, mul(k2, 0.5 * h, 2), 2), 2);
		k3[1] += stoerY(h * k + h / 2);

		k4 = mul(A, add(ynull, mul(k3, h, 2), 2), 2);
		k4[1] += stoerY(h * k + h);

		for (int i = 0; i < 2; i++) {
			s[i] = k1[i] + 2 * k2[i] + 2 * k3[i] + k4[i];
		}

		w = add(ynull, mul(s, h / 6, 2), 2);
		ynull = w;

		xy.getLast().y = w[0];
	}

}